#include "query.h"
#include "client.h"

#include <unity/scopes/Annotation.h>
#include <unity/scopes/CategorisedResult.h>
#include <unity/scopes/CategoryRenderer.h>
#include <unity/scopes/QueryBase.h>
#include <unity/scopes/SearchReply.h>

#include <iomanip>
#include <QDebug>

using namespace unity::scopes;
using namespace std;

const static string NEWS_RENDERER =
        R"(
{
        "schema-version": 1,
        "template": {
        "category-layout": "vertical-journal",
        "card-layout": "horizontal",
        "card-size": "medium",
        "overlay": false
        },
        "components": {
        "title": "title",
        "subtitle": "date",
        "art" : "art",
        "summary": "description"
        }
        }
        )";


Query::Query(const CannedQuery &query, const SearchMetadata &metadata) :
    SearchQueryBase(query, metadata) {
}

void Query::cancelled() {
    m_client.cancel();
}

void Query::run(SearchReplyProxy const& reply) {
    try {
        const CannedQuery &query(SearchQueryBase::query());
        // Trim the query string of whitespace
        //string query_string = alg::trim_copy(query.query_string());
        fetchQuery(reply, query);
    } catch (domain_error &e) {
        // Handle exceptions being thrown by the client API
        qDebug() << "EXCEPTION";
        qDebug() << e.what();
        reply->error(current_exception());
    }
}

void Query::fetchQuery(const SearchReplyProxy &reply, const CannedQuery &query) {
    Department::SPtr newest = move(Department::create("", query, "Ulm"));
    DepartmentList depList({ move(Department::create("index-reutlingen.xml", query, "Reutlingen")),
                                 move(Department::create("index-heidenheim.xml", query, "Heidenheim")),
                                 move(Department::create("index-gaildorf.xml", query, "Gaildorf")),
                                 move(Department::create("index-ehingen.xml", query, "Ehingen")),
                                 move(Department::create("index-crailsheim.xml", query, "Crailsheim")),
                                 move(Department::create("index-goeppingen.xml", query, "Göppingen")),
                                 move(Department::create("index-muensingen.xml", query, "Münsingen")),
                                 move(Department::create("index-geislingen.xml", query, "Geislingen")),
                                 move(Department::create("schwaebisch_hall/wechselaufmacher-schwaebisch_hall-direkt.xml", query, "Schwäbisch Hall")),
                                 move(Department::create("index-metzingen.xml", query, "Metzingen")),
                                 move(Department::create("index-bietigheim.xml", query, "Bietigheim-Bissingen")),
                                 move(Department::create("index-hechingen.xml", query, "Hechingen")),

                               });
    newest->set_subdepartments(depList);
    reply->register_departments(newest);

    Client::NewsList list;
    if (query.department_id().empty()) {
        list = m_client.allNews("", "index-ulm.xml", "");
    } else {
        list = m_client.allNews("", query.department_id(), "");
    }

    qDebug() << "query is" << QString::fromStdString(query.query_string());
    QString queryString = QString::fromStdString(query.query_string());
    Category::SCPtr cat = reply->register_category("unknown", "", "", CategoryRenderer(NEWS_RENDERER));
    QHash<QString, Category::SCPtr> categories;
    categories.insert("unknown", cat);
    for (const Client::NewsItem &news : list) {
        if (query.query_string().length() > 0
                && !news.title.toLower().contains(queryString.toLower())
                && !news.description.toLower().contains(queryString.toLower())
                ) {
            continue;
        }
        if (!categories.contains(news.category)) {
            Category::SCPtr newCat = reply->register_category(news.category.toStdString(), news.category.toStdString(), "", CategoryRenderer(NEWS_RENDERER));
            categories.insert(news.category, newCat);
        }

        if (!pushResult(reply, categories.value(news.category), news))
            return;
    }
}

bool Query::pushResult(const SearchReplyProxy &reply, Category::SCPtr category, const Client::NewsItem &news) {
    CategorisedResult res(category);
    res.set_title(news.title.toStdString());
    res.set_uri(news.link.toStdString());
    res.set_art(news.image.toStdString());
    res["date"] = news.date.toLocalTime().toString(Qt::DefaultLocaleShortDate).toStdString();
    res["subtitle"] = news.description.toStdString();
    res["description"] = news.description.toStdString();
    return reply->push(res);
}
