#ifndef SCOPE_SCOPE_H_
#define SCOPE_SCOPE_H_

#include <unity/scopes/ScopeBase.h>
#include <unity/scopes/QueryBase.h>
#include <unity/scopes/ReplyProxyFwd.h>
#include <unity/scopes/QueryBase.h>
#include <unity/scopes/PreviewQueryBase.h>

using namespace unity::scopes;

/**
 * Defines the lifecycle of scope plugin, and acts as a factory
 * for Query and Preview objects.
 *
 * Note that the #preview and #search methods are each called on
 * different threads, so some form of interlocking is required
 * if shared data structures are used.
 */
class MyScope: public ScopeBase {
public:
    /**
     * Called once at startup
     */
    void start(std::string const&) override;

    /**
     * Called at shutdown
     */
    void stop() override;

    /**
     * Called each time a new preview is requested
     */
    PreviewQueryBase::UPtr preview(const Result&, const ActionMetadata&) override;

    /**
     * Called each time a new query is requested
     */
    SearchQueryBase::UPtr search(CannedQuery const& q, SearchMetadata const&) override;
};

#endif // SCOPE_SCOPE_H_

